@@ -239,14 +239,36 @@ class WireProtocol(object):
         "Convert parameter array to BLR and values format."
         ln = len(params) * 2
         blr = bs([5, 2, 4, 0, ln & 255, ln >> 8])
-        values = bs([])
+        if self.accept_version < PROTOCOL_VERSION13:
+            values = bs([])
+        else:
+            # start with null indicator bitmap
+            null_indicator = 0
+            for i, p in enumerate(reversed(params)):
+                if p is None:
+                    null_indicator &= (1 << i)
+            n = len(params) // 8
+            if len(params) % 8 != 0:
+                n += 1
+            null_indicator_bytes = []
+            for i in range(n):
+                null_indicator_bytes.append(null_indicator & 255)
+                null_indicator >>= 8
+            if len(null_indicator_bytes) % 4:   # padding
+                null_indicator_bytes += [0] * (4 - len(null_indicator_bytes) % 4)
+            values = bs(null_indicator_bytes)
         for p in params:
-            t = type(p)
             if ((PYTHON_MAJOR_VER == 2 and type(p) == unicode) or
                 (PYTHON_MAJOR_VER == 3 and type(p) == str)):
                 p = self.str_to_bytes(p)
-                t = type(p)
-            if ((PYTHON_MAJOR_VER == 2 and t == str) or
+            t = type(p)
+            if p is None:
+                if self.accept_version < PROTOCOL_VERSION13:
+                    v = bs([])
+                    blr += bs([14, 0, 0])
+                else:
+                    continue
+            elif ((PYTHON_MAJOR_VER == 2 and t == str) or
                 (PYTHON_MAJOR_VER == 3 and t == bytes)):
                 if len(p) > MAX_CHAR_LENGTH:
                     v = self._create_blob(trans_handle, p)
@@ -290,21 +312,19 @@ class WireProtocol(object):
                 v = bs([1, 0, 0, 0]) if p else bs([0, 0, 0, 0])
                 blr += bs([23])
             else:   # fallback, convert to string
-                if p is None:
-                    v = bs([])
-                else:
-                    p = p.__repr__()
-                    if (PYTHON_MAJOR_VER==3 or
-                        (PYTHON_MAJOR_VER == 2 and type(p)==unicode)):
-                        p = self.str_to_bytes(p)
-                    v = p
+                p = p.__repr__()
+                if (PYTHON_MAJOR_VER==3 or
+                    (PYTHON_MAJOR_VER == 2 and type(p)==unicode)):
+                    p = self.str_to_bytes(p)
+                v = p
                 nbytes = len(v)
                 pad_length = ((4-nbytes) & 3)
                 v += bs([0]) * pad_length
                 blr += bs([14, nbytes & 255, nbytes >> 8])
-            values += v
             blr += bs([7, 0])
-            values += bs([0]) * 4 if p != None else bs([0xff,0xff,0xff,0xff])
+            values += v
+            if self.accept_version < PROTOCOL_VERSION13:
+                values += bs([0]) * 4 if p != None else bs([0xff,0xff,0xff,0xff])
         blr += bs([255, 76])    # [blr_end, blr_eoc]
         return blr, values
 
@@ -769,15 +789,35 @@ class WireProtocol(object):
         rows = []
         while count:
             r = [None] * len(xsqlda)
-            for i in range(len(xsqlda)):
-                x = xsqlda[i]
-                if x.io_length() < 0:
-                    b = self.recv_channel(4)
-                    ln = bytes_to_bint(b)
-                else:
-                    ln = x.io_length()
-                raw_value = self.recv_channel(ln, word_alignment=True)
-                if self.recv_channel(4) == bs([0]) * 4: # Not NULL
+            if self.accept_version < PROTOCOL_VERSION13:
+                for i in range(len(xsqlda)):
+                    x = xsqlda[i]
+                    if x.io_length() < 0:
+                        b = self.recv_channel(4)
+                        ln = bytes_to_bint(b)
+                    else:
+                        ln = x.io_length()
+                    raw_value = self.recv_channel(ln, word_alignment=True)
+                    if self.recv_channel(4) == bs([0]) * 4: # Not NULL
+                        r[i] = x.value(raw_value)
+            else:   # PROTOCOL_VERSION13
+                n = len(xsqlda) // 8
+                if len(xsqlda) % 8 != 0:
+                    n += 1
+                null_indicator = 0
+                for c in reversed(self.recv_channel(n, word_alignment=True)):
+                    null_indicator <<=8
+                    null_indicator += c if PYTHON_MAJOR_VER == 3 else ord(c)
+                for i in range(len(xsqlda)):
+                    x = xsqlda[i]
+                    if null_indicator & (1 << i):
+                        continue
+                    if x.io_length() < 0:
+                        b = self.recv_channel(4)
+                        ln = bytes_to_bint(b)
+                    else:
+                        ln = x.io_length()
+                    raw_value = self.recv_channel(ln, word_alignment=True)
                     r[i] = x.value(raw_value)
             rows.append(r)
             b = self.recv_channel(12)
@@ -951,18 +991,39 @@ class WireProtocol(object):
         r = []
         if count == 0:
             return []
-        for i in range(len(xsqlda)):
-            x = xsqlda[i]
-            if x.io_length() < 0:
-                b = self.recv_channel(4)
-                ln = bytes_to_bint(b)
-            else:
-                ln = x.io_length()
-            raw_value = self.recv_channel(ln, word_alignment=True)
-            if self.recv_channel(4) == bs([0]) * 4: # Not NULL
-                r.append(x.value(raw_value))
-            else:
-                r.append(None)
+        if self.accept_version < PROTOCOL_VERSION13:
+            for i in range(len(xsqlda)):
+                x = xsqlda[i]
+                if x.io_length() < 0:
+                    b = self.recv_channel(4)
+                    ln = bytes_to_bint(b)
+                else:
+                    ln = x.io_length()
+                raw_value = self.recv_channel(ln, word_alignment=True)
+                if self.recv_channel(4) == bs([0]) * 4: # Not NULL
+                    r.append(x.value(raw_value))
+                else:
+                    r.append(None)
+        else:
+            n = len(xsqlda) // 8
+            if len(xsqlda) % 8 != 0:
+                n += 1
+            null_indicator = 0
+            for c in reversed(self.recv_channel(n, word_alignment=True)):
+                null_indicator <<=8
+                null_indicator += c if PYTHON_MAJOR_VER == 3 else ord(c)
+            for i in range(len(xsqlda)):
+                x = xsqlda[i]
+                if null_indicator & (1 << i):
+                    r.append(None)
+                else:
+                    if x.io_length() < 0:
+                        b = self.recv_channel(4)
+                        ln = bytes_to_bint(b)
+                    else:
+                        ln = x.io_length()
+                    raw_value = self.recv_channel(ln, word_alignment=True)
+                    r.append(x.value(raw_value))
         return r
 
     def _wait_for_event(self, timeout):
